@@ -7,6 +7,8 @@
 //! [`ivt`]: crate::ivt
 //! [`ast`]: crate::ast
 
+use regex::RegexBuilder;
+
 use crate::ast;
 use crate::ivt::*;
 use crate::parser::{parse_cddl, slice_parse_cddl};
@@ -18,7 +20,7 @@ pub type FlattenResult<T> = std::result::Result<T, ValidateError>;
 
 /// Convert a CDDL schema in UTF-8 form into a structured rule set.
 pub fn flatten_from_str(cddl_input: &str) -> FlattenResult<RulesByName> {
-    let cddl = parse_cddl(cddl_input).map_err(ValidateError::ParseError)?;
+    let cddl = parse_cddl(cddl_input)?;
     flatten(&cddl)
 }
 
@@ -27,14 +29,14 @@ pub fn flatten_from_str(cddl_input: &str) -> FlattenResult<RulesByName> {
 /// This works the same as `flatten_from_str`, but preserves a copy of the original
 /// CDDL text alongside the IVT.
 pub fn slice_flatten_from_str(cddl_input: &str) -> FlattenResult<RulesWithStrings> {
-    let cddl = slice_parse_cddl(cddl_input).map_err(ValidateError::ParseError)?;
+    let cddl = slice_parse_cddl(cddl_input)?;
     slice_flatten(&cddl)
 }
 
 /// Convert an already-parsed cddl AST into a `(name, rules)` map.
 pub fn flatten(cddl: &ast::Cddl) -> FlattenResult<RulesByName> {
     // This first pass generates a tree of Nodes from the AST.
-    cddl.rules.iter().map(|rule| flatten_rule(rule)).collect()
+    cddl.rules.iter().map(flatten_rule).collect()
 }
 
 /// Convert an already-parsed cddl AST into a `(name, (rule, rule-string))` map.
@@ -71,7 +73,7 @@ fn flatten_rule(rule: &ast::Rule) -> FlattenResult<(String, RuleDef)> {
 }
 
 fn flatten_type(ty: &ast::Type) -> FlattenResult<Node> {
-    let options: FlattenResult<Vec<Node>> = ty.0.iter().map(|type1| flatten_type1(type1)).collect();
+    let options: FlattenResult<Vec<Node>> = ty.0.iter().map(flatten_type1).collect();
     let options = options?;
 
     match options.len() {
@@ -92,11 +94,8 @@ fn flatten_type1(ty1: &ast::Type1) -> FlattenResult<Node> {
     }
 }
 
-// We only support the following control operators:
-//
-// <target> .size <integer literal>
-//
-//     The only allowed targets are bstr, tstr, and unsigned integers.
+// Control operators are functions that further restrict an underlying
+// type.
 //
 // TODO:
 // .bits
@@ -108,30 +107,71 @@ fn flatten_type1(ty1: &ast::Type1) -> FlattenResult<Node> {
 //
 fn flatten_control(ctl: &ast::TypeControl) -> FlattenResult<Node> {
     let ctl_result = match &ctl.op[..] {
-        "size" => {
-            let target = flatten_type2(&ctl.target)?;
-            let size = flatten_type2(&ctl.arg)?;
-
-            // The only allowed limit types are:
-            // A positive literal integer
-            // A named rule (which should resolve to a literal integer)
-            match size {
-                Node::Literal(Literal::Int(_)) => {}
-                Node::Rule(_) => {}
-                _ => return Err(ValidateError::Unsupported(".size limit type".into())),
-            };
-
-            Control::Size(CtlOpSize {
-                target: Box::new(target),
-                size: Box::new(size),
-            })
-        }
+        "size" => control_size(ctl)?,
+        "regexp" => control_regex(ctl)?,
         _ => return Err(ValidateError::Unsupported("control operator".into())),
     };
 
     Ok(Node::Control(ctl_result))
 }
 
+// Handle the "size" control operator:
+// <target> .size <integer literal>
+// The only allowed targets are bstr, tstr, and unsigned integers.
+//
+fn control_size(ctl: &ast::TypeControl) -> FlattenResult<Control> {
+    let target = flatten_type2(&ctl.target)?;
+    let size = flatten_type2(&ctl.arg)?;
+
+    // The only allowed limit types are:
+    // A positive literal integer
+    // A named rule (which should resolve to a literal integer)
+    match size {
+        Node::Literal(Literal::Int(_)) => {}
+        Node::Rule(_) => {}
+        _ => return Err(ValidateError::Unsupported(".size limit type".into())),
+    };
+
+    Ok(Control::Size(CtlOpSize {
+        target: Box::new(target),
+        size: Box::new(size),
+    }))
+}
+
+// Handle the "size" control operator:
+// <target> .size <integer literal>
+// The only allowed targets are bstr, tstr, and unsigned integers.
+//
+fn control_regex(ctl: &ast::TypeControl) -> FlattenResult<Control> {
+    let target = flatten_type2(&ctl.target)?;
+    let regexp_node = flatten_type2(&ctl.arg)?;
+
+    // The target type must be a text string.
+    match target {
+        Node::PreludeType(PreludeType::Tstr) => {}
+        _ => {
+            return Err(ValidateError::Structural("bad regexp target type".into()));
+        }
+    }
+
+    // The regular expression itself should be a literal string.
+    match regexp_node {
+        Node::Literal(Literal::Text(re_str)) => {
+            // Compile the regex.
+            // Limit the size of the result, so that untrusted input can't
+            // use a lot of memory. The regex crate says this will adequately
+            // protect against using too much CPU time as well.
+            let re = RegexBuilder::new(&re_str)
+                .size_limit(1 << 20)
+                .build()
+                .map_err(|_| ValidateError::Structural("malformed regexp".into()))?;
+
+            Ok(Control::Regexp(CtlOpRegexp { re }))
+        }
+        _ => Err(ValidateError::Structural("improper regexp type".into())),
+    }
+}
+
 // The only way a range start or end can be specified is with a literal
 // value, or with a typename.  We will accept either of those, and throw
 // an error otherwise.  Let the validator worry about whether a typename
@@ -351,10 +391,7 @@ fn flatten_group(group: &ast::Group) -> FlattenResult<Vec<Node>> {
 
 fn flatten_groupchoice(groupchoice: &ast::GrpChoice) -> FlattenResult<Vec<Node>> {
     let group_entries = &groupchoice.0;
-    let kvs: FlattenResult<Vec<Node>> = group_entries
-        .iter()
-        .map(|group_entry| flatten_groupentry(group_entry))
-        .collect();
+    let kvs: FlattenResult<Vec<Node>> = group_entries.iter().map(flatten_groupentry).collect();
     kvs
 }
 
