@@ -109,7 +109,7 @@
 //! - Extend group with `//=`
 //! - Type sockets with `$`
 //! - Group sockets with `$$`
-//! - Control operators other than those above (e.g. `.bits`, `lt`, `gt`...)
+//! - Control operators other than those above (e.g. `.bits`, `.lt`, `.gt`...)
 //! - Group enumeration with `&`
 //! - Tagged data with `#`
 //! - Hexfloat literals (e.g. `0x1.921fb5p+1`)
@@ -121,10 +121,6 @@
 #![warn(missing_docs)]
 #![forbid(unsafe_code)]
 #![warn(clippy::cast_possible_truncation)]
-#![allow(
-    // matches! is only 1.42+
-    clippy::match_like_matches_macro,
-)]
 
 pub mod ast;
 pub mod context;
